import 'dart:convert';
import 'package:lokalise_flutter_sdk/src/extensions/string_locale.dart';
import 'package:lokalise_flutter_sdk/src/generator/generator_config.dart';
import 'package:lokalise_flutter_sdk/src/generator/intl_translation_proxy.dart';
import 'package:lokalise_flutter_sdk/src/generator/utils/file_utils.dart';
import 'package:lokalise_flutter_sdk/src/generator/utils/utils.dart';
import 'generator_exception.dart';
import 'label.dart';
import 'templates.dart';

class Generator {
  final GeneratorConfig _config;

  Generator({required GeneratorConfig config}) : _config = config;

  /// Generator
  Future<void> generateAsync() async {
    await _updateL10nDir();
    await _updateGeneratedDir();
    await _generateDartFiles();
  }

  /// Updater
  Future<void> _updateL10nDir() async {
    var mainArbFile = getArbFileForLocale(_config.mainLocale, _config.arbDir);
    if (mainArbFile == null) {
      await createArbFileForLocale(_config.mainLocale, _config.arbDir);
    }
  }

  /// Update generated folder
  Future<void> _updateGeneratedDir() async {
    final labels = _getLabelsFromMainArbFile();
    List<String> locales = getLocales(_config.arbDir);
    locales = _orderLocales(locales);
    for (String locale in locales) {
      if (!locale.isValidLocale) {
        throw GeneratorException('Invalid locale: $locale');
      }
    }

    final content = generateL10nDartFileContent(
      _config.outputClass,
      labels,
      locales,
      _config.otaEnabled,
    );
    final formattedContent = formatDartContent(content, 'l10n.dart');
    await updateL10nDartFile(formattedContent, _config.outputDir);

    final intlDir = getIntlDirectory(_config.outputDir);
    if (intlDir == null) {
      await createIntlDirectory(_config.outputDir);
    }
    await removeUnusedGeneratedDartFiles(locales, _config.outputDir);
  }

  /// Get data from arb file
  List<Label> _getLabelsFromMainArbFile() {
    var mainArbFile = getArbFileForLocale(_config.mainLocale, _config.arbDir);
    if (mainArbFile == null) {
      throw GeneratorException(
          "Can't find ARB file for the '${_config.mainLocale}' locale.");
    }
    var content = mainArbFile.readAsStringSync();
    var decodedContent = json.decode(content) as Map<String, dynamic>;
    var labels =
        decodedContent.keys.where((key) => !key.startsWith('@')).map((key) {
      var name = key;
      var content = decodedContent[key];
      var meta = decodedContent['@$key'] ?? {};
      var type = meta['type'];
      var description = meta['description'];
      var placeholders = meta['placeholders'] != null
          ? (meta['placeholders'] as Map<String, dynamic>)
              .keys
              .map((placeholder) => Placeholder(
                  key, placeholder, meta['placeholders'][placeholder]))
              .toList()
          : null;

      return Label(name, content,
          type: type, description: description, placeholders: placeholders);
    }).toList();
    return labels;
  }

  /// Locales order
  List<String> _orderLocales(List<String> locales) {
    var index = locales.indexOf(_config.mainLocale);
    return index != -1
        ? [
            locales.elementAt(index),
            ...locales.sublist(0, index),
            ...locales.sublist(index + 1)
          ]
        : locales;
  }

  /// Generate dart files
  Future<void> _generateDartFiles() async {
    var outputDir = getIntlDirectoryPath(_config.outputDir);
    var dartFile = getL10nDartFilePath(_config.outputDir);
    var arbFiles =
        getArbFiles(_config.arbDir).map((file) => file.path).toList();
    var proxy = IntlTranslationProxy();
    proxy.generateFromArb(
        outputDir: outputDir, dartFile: dartFile, arbFiles: arbFiles);
  }
}
