/// This class is based on
/// https://github.com/dart-lang/intl_translation/blob/e27424105c3380297ce564a95771446080181e48/bin/extract_to_arb.dart
///
/// We need to create a proxy to connect with the intl_translation library
/// due it hasn't got any 'public' method to call to the generator, as it
/// is a command.
/// Also, we can't execute the command directly from our code using
/// Process.run due to flutter doesn't let to execute commands on transitive
/// dependencies and we need to control the version of intl_translation that we are
/// using.
///
/// Important Note: this file need to be revised after update the intl_translation
/// version to adapt to the new version of the library, please check the new
/// version of generate_from_arb.dart from the library to understand what changes
/// should you apply here.

// ignore_for_file: implementation_imports

import 'dart:convert';
import 'dart:io';
import 'package:path/path.dart' as path;
import 'package:intl_translation/extract_messages.dart';
import 'package:intl_translation/generate_localized.dart';
import 'package:intl_translation/src/message_parser.dart';
import 'package:intl_translation/src/messages/message.dart';
import 'package:intl_translation/src/messages/literal_string_message.dart';
import 'package:intl_translation/src/messages/main_message.dart';

class IntlTranslationProxy {
  final JsonCodec jsonDecoder = const JsonCodec();
  final MessageExtraction extraction = MessageExtraction();
  final MessageGeneration generation = MessageGeneration();

  IntlTranslationProxy() {
    extraction.suppressWarnings = true;
    generation.useDeferredLoading = false;
    generation.generatedFilePrefix = '';
  }

  void generateFromArb(
      {required String outputDir,
      required String dartFile,
      required List<String> arbFiles}) {
    Map<String, MainMessage> allMessages = extraction.parseFile(File(dartFile));
    Map<String, List<MainMessage>> messages = {};
    allMessages.forEach(
        (key, value) => messages.putIfAbsent(key, () => []).add(value));

    _generateLocaleFiles(arbFiles, outputDir, messages);

    var mainImportFile = File(path.join(
        outputDir, '${generation.generatedFilePrefix}messages_all.dart'));
    mainImportFile.writeAsStringSync(generation.generateMainImportFile());

    var localesImportFile = File(path.join(outputDir,
        '${generation.generatedFilePrefix}messages_all_locales.dart'));
    localesImportFile.writeAsStringSync(generation.generateLocalesImportFile());
  }

  void _generateLocaleFiles(List<String> arbFiles, String outputDir,
      Map<String, List<MainMessage>> messages) {
    for (String arbFile in arbFiles) {
      File file = File(arbFile);
      Map<String, dynamic> parsedArb =
          jsonDecoder.decode(file.readAsStringSync());

      String? locale = parsedArb['@@locale'] ?? parsedArb['_locale'];
      if (locale == null) {
        // Get the locale from the end of the file name. This assumes that the file
        // name doesn't contain any underscores except to begin the language tag
        // and to separate language from country. Otherwise we can't tell if
        // my_file_fr.arb is locale "fr" or "file_fr".
        String name = path.basenameWithoutExtension(file.path);
        locale = name.split('_').skip(1).join('_');
      }

      // Remove all metadata from the map
      parsedArb.remove('_locale');
      parsedArb.removeWhere((key, _) => key.startsWith('@'));

      List<TranslatedMessage> translations =
          Map.castFrom<String, dynamic, String, String>(parsedArb)
              .entries
              .map((e) => _recreateIntlObjects(e.key, e.value, messages))
              .toList();

      generation.allLocales.add(locale);
      generation.generateIndividualMessageFile(locale, translations, outputDir);
    }
  }

  TranslatedMessage _recreateIntlObjects(
    String id,
    String data,
    Map<String, List<MainMessage>> messages,
  ) {
    MessageParser messageParser = MessageParser(data);
    Message parsed = messageParser.pluralGenderSelectParse();
    if (parsed is LiteralString && parsed.string.isEmpty) {
      parsed = messageParser.nonIcuMessageParse();
    }
    return TranslatedMessage(id, parsed, messages[id] ?? []);
  }
}
