// ignore: implementation_imports
import 'package:intl/src/intl_helpers.dart';
import 'package:intl/intl.dart';
import 'package:intl/message_lookup_by_library.dart';
import 'package:lokalise_flutter_sdk/src/ota/domain/models/bundle.dart';
import 'package:lokalise_flutter_sdk/src/ota/domain/models/language_bundle.dart';

class MessageLookupProxy implements MessageLookup {
  final MessageLookup _messageLookup;
  Map<String, List<String>>? _argumentNamesForKeyNames;
  Bundle _bundle;

  MessageLookupProxy({
    required MessageLookup messageLookup,
    required Bundle bundle,
  })  : _messageLookup = (messageLookup is UninitializedLocaleData)
            ? CompositeMessageLookup()
            : messageLookup,
        _bundle = bundle;

  int get translationVersion => _bundle.translationVersion;

  set bundle(Bundle bundle) {
    _bundle = bundle;
  }

  set argumentNamesForKeyNames(Map<String, List<String>> param) {
    _argumentNamesForKeyNames = param;
  }

  @override
  void addLocale(String localeName, Function findLocale) {
    _messageLookup.addLocale(localeName, findLocale);
  }

  @override
  String? lookupMessage(
    String? messageText,
    String? locale,
    String? name,
    List<Object>? args,
    String? meaning, {
    MessageIfAbsent? ifAbsent,
  }) {
    final currentLocale = locale ?? Intl.getCurrentLocale();
    final translations = _bundle.languageBundles
        .firstWhere(
          (e) => e.locale.toLowerCase() == currentLocale.toLowerCase(),
          orElse: () => LanguageBundle(locale: currentLocale, translations: {}),
        )
        .translations;

    String? message;
    if (translations.containsKey(name)) {
      final translationArgs = _mapArgs(name, args);
      if (translationArgs != null) {
        message = translations[name]?.getTranslation(args: translationArgs);
      }
    }

    return message ??
        _messageLookup.lookupMessage(
          messageText,
          locale,
          name,
          args,
          meaning,
          ifAbsent: ifAbsent,
        );
  }

  Map<String, Object>? _mapArgs(String? keyName, List<Object>? argsValues) {
    // in metadata we have the names of the arguments for the translation
    // so here we are getting it using the keyName, note that in theory,
    // it can be null
    final argNames = _argumentNamesForKeyNames?[keyName];

    if (argNames == null && argsValues == null) {
      // If both are nulls we consider it as empty,
      // which means no arguments for the translation
      return <String, Object>{};
    } else if (argNames?.length == argsValues?.length) {
      // if both have the same length, it is also checking that both
      // are not null, we can build the result map {argName: argValue}
      return Map.fromIterables(argNames!, argsValues!);
    }

    // if we arrive here means that there is something wrong with
    // the arguments for the key so we can't build them and we just
    // return a null and the caller should control what happens in this case.
    return null;
  }
}
