import 'package:flutter_test/flutter_test.dart';
import 'package:lokalise_flutter_sdk/src/ota/data/api/clients/ota/response/error/ota_error_response.dart';
import 'package:lokalise_flutter_sdk/src/ota/domain/exceptions/validation_exception.dart';

void main() {
  group('OtaErrorResponse - fromJson', () {
    test('empty json', () {
      // Given
      final Map<String, dynamic> json = {};

      // When
      ValidationException? exception;
      try {
        OtaErrorResponse.fromJson(json: json);
      } on ValidationException catch (e) {
        exception = e;
      }

      // Then
      expect(exception, isNotNull);
      expect(exception!.errors.length, equals(3));
    });

    test('statusCode invalid type', () {
      // Given
      final Map<String, dynamic> json = {
        'statusCode': '',
        'message': '',
        'error': [],
      };

      // When
      ValidationException? exception;
      try {
        OtaErrorResponse.fromJson(json: json);
      } on ValidationException catch (e) {
        exception = e;
      }

      // Then
      expect(exception, isNotNull);
      expect(exception!.errors.length, equals(1));
      final error = exception.errors[0];
      expect(error.type, equals(OtaErrorResponse));
      expect(error.path, equals('statusCode'));
      expect(error.received, equals(''));
      expect(error.expectedType, equals(int));
    });

    test('message invalid type', () {
      // Given
      final Map<String, dynamic> json = {
        'statusCode': 200,
        'message': 1,
        'error': [],
      };

      // When
      ValidationException? exception;
      try {
        OtaErrorResponse.fromJson(json: json);
      } on ValidationException catch (e) {
        exception = e;
      }

      // Then
      expect(exception, isNotNull);
      expect(exception!.errors.length, equals(1));
      final error = exception.errors[0];
      expect(error.type, equals(OtaErrorResponse));
      expect(error.path, equals('message'));
      expect(error.received, equals(1));
      expect(error.expectedType, equals(int));
    });

    test('error invalid type', () {
      // Given
      final Map<String, dynamic> json = {
        'statusCode': 200,
        'message': '',
        'error': '',
      };

      // When
      ValidationException? exception;
      try {
        OtaErrorResponse.fromJson(json: json);
      } on ValidationException catch (e) {
        exception = e;
      }

      // Then
      expect(exception, isNotNull);
      expect(exception!.errors.length, equals(1));
      final error = exception.errors[0];
      expect(error.type, equals(OtaErrorResponse));
      expect(error.path, equals('error'));
      expect(error.received, equals(''));
      expect(error.expectedType, equals(List));
    });

    test('correct, errors is empty', () {
      // Given
      final Map<String, dynamic> json = {
        'statusCode': 200,
        'message': 'test-message',
        'error': [],
      };

      // When
      ValidationException? exception;
      OtaErrorResponse? response;
      try {
        response = OtaErrorResponse.fromJson(json: json);
      } on ValidationException catch (e) {
        exception = e;
      }

      // Then
      expect(exception, isNull);
      expect(response, isNotNull);
      expect(response!.statusCode, equals(json['statusCode']));
      expect(response.message, equals(json['message']));
      expect(response.errors, isEmpty);
    });

    test('correct', () {
      // Given
      final Map<String, dynamic> json = {
        'statusCode': 200,
        'message': 'test-message',
        'error': [
          {
            'code': 'test-code',
            'message': 'test-message',
            'path': [],
          }
        ],
      };

      // When
      ValidationException? exception;
      OtaErrorResponse? response;
      try {
        response = OtaErrorResponse.fromJson(json: json);
      } on ValidationException catch (e) {
        exception = e;
      }

      // Then
      expect(exception, isNull);
      expect(response, isNotNull);
      expect(response!.statusCode, equals(json['statusCode']));
      expect(response.message, equals(json['message']));
      expect(response.errors.length, equals(1));
      final error = response.errors[0];
      expect(error.code, equals(json['error'][0]['code']));
      expect(error.message, equals(json['error'][0]['message']));
      expect(error.paths, isEmpty);
    });
  });
}
