import 'package:flutter_test/flutter_test.dart';
import 'package:lokalise_flutter_sdk/src/ota/domain/data_source/local_bundle_data_source.dart';
import 'package:lokalise_flutter_sdk/src/ota/domain/models/bundle.dart';
import 'package:lokalise_flutter_sdk/src/ota/domain/use_cases/get_local_bundle_use_case.dart';
import 'package:mockito/annotations.dart';
import 'package:mockito/mockito.dart';

import 'get_local_bundle_use_case_test.mocks.dart';

@GenerateMocks([LocalBundleDataSource])
void main() {
  late MockLocalBundleDataSource localDataSource;
  late GetLocalBundleUseCase useCase;

  setUp(() {
    localDataSource = MockLocalBundleDataSource();
    useCase = GetLocalBundleUseCase(localBundleDataSource: localDataSource);
  });

  group('GetLocalBundleUseCase - getBundle', () {
    test('repo throw exception', () async {
      // Given
      when(localDataSource.getBundle()).thenThrow(Exception());

      // When
      const projectId = 'test-project';
      const appVersion = 'test-appVersion';
      final res = await useCase.getBundle(
        projectId: projectId,
        appVersion: appVersion,
      );

      // Then
      expect(res.projectId, equals(projectId));
      expect(res.appVersion, equals(appVersion));
      expect(res.translationVersion, equals(0));
      expect(res.languageBundles, isEmpty);
    });

    test('repo returns null', () async {
      // Given
      when(localDataSource.getBundle()).thenReturn(null);
      when(localDataSource.removeBundle()).thenThrow(Exception());

      // When
      const projectId = 'test-project';
      const appVersion = 'test-appVersion';
      final res = await useCase.getBundle(
        projectId: projectId,
        appVersion: appVersion,
      );

      // Then
      expect(res.projectId, equals(projectId));
      expect(res.appVersion, equals(appVersion));
      expect(res.translationVersion, equals(0));
      expect(res.languageBundles, isEmpty);
    });

    test('repo returns bundle with a different project id', () async {
      // Given
      const appVersion = 'test-appVersion';
      final bundle = Bundle(
        projectId: 'another',
        translationVersion: 1,
        appVersion: appVersion,
        languageBundles: [],
      );
      when(localDataSource.getBundle()).thenReturn(bundle);
      when(localDataSource.removeBundle())
          .thenAnswer((_) => Future.value(true));

      // When
      const projectId = 'test-projectId';
      final res = await useCase.getBundle(
        projectId: projectId,
        appVersion: appVersion,
      );

      // Then
      expect(res.projectId, equals(projectId));
      expect(res.appVersion, equals(appVersion));
      expect(res.translationVersion, equals(0));
      expect(res.languageBundles, isEmpty);
    });

    test('repo returns bundle with a different app version', () async {
      // Given
      const projectId = 'test-projectId';
      final bundle = Bundle(
        projectId: projectId,
        translationVersion: 1,
        appVersion: 'another',
        languageBundles: [],
      );
      when(localDataSource.getBundle()).thenReturn(bundle);
      when(localDataSource.removeBundle())
          .thenAnswer((_) => Future.value(true));

      // When
      const appVersion = 'test-appVersion';
      final res = await useCase.getBundle(
        projectId: projectId,
        appVersion: appVersion,
      );

      // Then
      expect(res.projectId, equals(projectId));
      expect(res.appVersion, equals(appVersion));
      expect(res.translationVersion, equals(0));
      expect(res.languageBundles, isEmpty);
    });

    test('success', () async {
      // Given
      const projectId = 'test-projectId';
      const appVersion = 'test-appVersion';
      final bundle = Bundle(
        projectId: projectId,
        translationVersion: 1,
        appVersion: appVersion,
        languageBundles: [],
      );
      when(localDataSource.getBundle()).thenReturn(bundle);
      when(localDataSource.removeBundle())
          .thenAnswer((_) => Future.value(true));

      // When
      final res = await useCase.getBundle(
        projectId: projectId,
        appVersion: appVersion,
      );

      // Then
      expect(res, equals(bundle));
    });
  });
}
